useDynLib(DEoptim, .registration = TRUE)
export(DEoptim, DEoptim.control)
S3method("plot", "DEoptim")
S3method("summary", "DEoptim")
importFrom("grDevices", "devAskNewPage")
importFrom("graphics", "abline", "matplot", "par", "plot", "plot.new")
importFrom("stats", "runif")
importFrom("parallel", "clusterExport", "parApply", "makeCluster",
           "clusterCall", "stopCluster")
importFrom("methods", "hasArg")
importFrom("utils", "installed.packages")