# DEoptim
[![CRAN](https://www.r-pkg.org/badges/version/DEoptim)](https://cran.r-project.org/package=DEoptim)
[![Downloads](https://cranlogs.r-pkg.org/badges/DEoptim?color=brightgreen)](https://www.r-pkg.org/pkg/DEoptim)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/DEoptim?color=brightgreen)](https://www.r-pkg.org/pkg/DEoptim)

`DEoptim` ([Ardia et al., 20xx](https://CRAN.R-project.org/package=DEoptim)) implements the Differential Evolution algorithm for global optimization of a real-valued function of a real-valued parameter vector. The implementation of Differential Evolution in DEoptim interfaces with C
code for efficiency.  

The latest stable version of `DEoptim` is available at [https://cran.r-project.org/package=DEoptim](https://cran.r-project.org/package=DEoptim).

The latest development version of `DEoptim` is available at [https://github.com/ArdiaD/DEoptim](https://github.com/ArdiaD/DEoptim).

## Please cite the package in publications!

By using `DEoptim` you agree to the following rules: 

1) You must cite [Mullen et al. (2011)](https://doi.org//10.18637/jss.v040.i06) in working papers and published papers that use `DEoptim`.
2) You must place the following URL in a footnote to help others find `DEoptim`: [https://CRAN.R-project.org/package=DEoptim](https://CRAN.R-project.org/package=DEoptim). 
3) You assume all risk for the use of `DEoptim`.

Mullen, K., Ardia, D., Gil, D., Windover, D., Cline, J. (2011).  
DEoptim: An R package for global optimization by Differential Evolution.  
_Journal of Statistical Software_, 40(6), 1-26.  
[https://doi.org//10.18637/jss.v040.i06](https://doi.org//10.18637/jss.v040.i06)

## Other references

Ardia, D., Mullen, K., Peterson, B.G., Ulrich, J. (20xx).  
_DEoptim: Differential Evolution in R_.  
[https://CRAN.R-project.org/package=DEoptim](https://CRAN.R-project.org/package=DEoptim)

Ardia, D., Boudt, K., Carl, P., Mullen, K., Peterson, B.G. (2010).  
Differential Evolution with DEoptim: An application to non-convex portfolio optimization.  
_R Journal_, 3(1), 27-34.  
[https://doi.org/10.32614/RJ-2011-005](https://doi.org/10.32614/RJ-2011-005)
