# DEoptim

`DEoptim` implements the Differential Evolution algorithm for global optimization of a real-valued function of a
real-valued parameter vector. The implementation of Differential Evolution in DEoptim interfaces with C
code for efficiency.  Moreover, the package is self-contained and does not depend on any other packages.

## Please cite [Mullen et al. (2011)](https://doi.org//10.18637/jss.v040.i06) in working papers and published papers that use `DEoptim`.

## Papers discussing the package and its applications:  

Mullen, K., Ardia, D., Gil, D., Windover, D., Cline, J. (2011).  
DEoptim: An R package for global optimization by Differential Evolution.  
_Journal of Statistical Software_, 40(6), 1-26.  
[https://doi.org//10.18637/jss.v040.i06](https://doi.org//10.18637/jss.v040.i06)

Ardia, D., Boudt, K., Carl, P., Mullen, K., Peterson, B.G. (2010).  
Differential Evolution with DEoptim: An application to non-convex portfolio optimization.  
_R Journal_, 3(1), 27-34.  
[https://doi.org/10.32614/RJ-2011-005](https://doi.org/10.32614/RJ-2011-005)
